--[[
	Copyright (C) Udorn (Blackhand)
--]]

local viewType = "AchievementEntry"
local version = 1
local YakaGui = LibStub("YakaGui", true)
local info = YakaGui:GetViewInfo(viewType) 
if (info and info.version >= version) then 
	return 
end

qmstr = qmstr or {}
qmstr.AchievementEntry = {}
qmstr.AchievementEntry.prototype = {}
qmstr.AchievementEntry.metatable = {__index = qmstr.AchievementEntry.prototype}
setmetatable(qmstr.AchievementEntry.prototype, {__index = yaka.Container.prototype})

qmstr.AchievementEntry.info = {viewType = viewType, version = version}
qmstr.AchievementEntry.GetViewInfo = function()
	return qmstr.AchievementEntry.info
end

qmstr.AchievementEntry.textViews = {}
qmstr.AchievementEntry.checkboxViews = {}

local ICON_SIZE = 24
local ICON_PAD_TOP = 1

local log = vendor.Debug:new("AchievementEntry")

local function _OnClick(info)
	local self = info.arg
	local achievementId = self.cfg.achievementId
	
	if (not AchievementFrame) then
		AchievementFrame_LoadUI()
	end
	
	if (IsModifiedClick("CHATLINK") and ChatEdit_GetActiveWindow()) then
		local achievementLink = GetAchievementLink(achievementId)
		if ( achievementLink ) then
			ChatEdit_InsertLink(achievementLink)
		end
	elseif (IsModifiedClick("QUESTWATCHTOGGLE")) then
		WatchFrame_StopTrackingAchievement(info.button, achievementId)
	elseif (not AchievementFrame:IsShown()) then
		AchievementFrame_ToggleAchievementFrame()
		AchievementFrame_SelectAchievement(achievementId)
	else
		if (AchievementFrameAchievements.selection ~= achievementId) then
			AchievementFrame_SelectAchievement(achievementId)
		else
			AchievementFrame_ToggleAchievementFrame()
		end
	end		
end

local function _SetActive(info)
	qmstr.QuestMaster:SetIgnoreAchievement(info.view:GetId(), not info.value)
end

local function _IsActive(info)
	return not qmstr.QuestMaster:IsAchievementIgnored(info.view:GetId())
end


local function _WhipeChildViews(container)
	for k,v in pairs(container:GetChildViews()) do
		if ("Text" == v:GetViewType()) then
			table.insert(qmstr.AchievementEntry.textViews, v)
		elseif ("Checkbox" == v:GetViewType()) then
			table.insert(qmstr.AchievementEntry.checkboxViews, v)
		end
	end
	container:RemoveChildViews()
end

local function _GetText(self, text, height, tooltip, paddingLeft, paddingTop)
	local order = self.order
	self.order = self.order + 1
	if (#qmstr.AchievementEntry.textViews > 0) then
		local view = table.remove(qmstr.AchievementEntry.textViews)
		view:SetText(text)
		view:SetArg(self)
		view:SetHeight(height)
		view:SetOrder(order)
		view:SetPadding(paddingLeft, 0, paddingTop, 0)
		view:SetTooltip(tooltip)
		return view
	end
	return YakaGui:CreateView({
		viewType = "Text",
		text = text,
		order = order,
		height = height,
		width = "fill",
		paddingTop = paddingTop,
		paddingLeft = paddingLeft,
		onClick = _OnClick,
		tooltip = tooltip,
		font = "QuestMasterNormal",
		arg = self
	})
end

local function _GetCheckbox(self, achievementId)
	local order = self.order
	self.order = self.order + 1
	if (#qmstr.AchievementEntry.checkboxViews > 0) then
		local view = table.remove(qmstr.AchievementEntry.checkboxViews)
		view:SetOrder(order)
		view:SetId(achievementId)
		view:SetChecked(not qmstr.QuestMaster:IsAchievementIgnored(achievementId))
		return view
	end
	return YakaGui:CreateView({
		viewType = "Checkbox",
		order = order,
		height = 16,
		width = 16,
		id = achievementId,
		getValue = _IsActive,
		setValue = _SetActive,
		arg = self,
	})
end
local function _MergeColor(src, dest, percent)
	local diff = src - dest
	local rtn = src - (diff * percent)
	return math.min(1, math.max(0, rtn))
end

local function _ColorizeObjective(current, total)
	local min = RED_FONT_COLOR
	local max = GREEN_FONT_COLOR
	local color = {r = 1, g = 1, b = 1, a = 1}
	if (current and total) then
		current = tonumber(current)
		total = tonumber(total)
		local percent = 0
		if (current > 0) then
			percent = current / total
		end
		color.r = _MergeColor(min.r, max.r, percent) 
		color.g = _MergeColor(min.g, max.g, percent)
		color.b = _MergeColor(min.b, max.b, percent)
	end
	return YakaGui:GetFontColorCode(color)
end

--local function _IsSummarizeable(achievementId)
--	local rtn = true
--	local numCriteria = GetAchievementNumCriteria(achievementId)
--	for i = 1, numCriteria do
--		local _, criteriaType, criteriaCompleted, _, totalQuantity = GetAchievementCriteriaInfo(achievementId, i)
--		if (not criteriaCompleted and (totalQuantity > 1 and criteriaType != 46)) then
--			rtn = false
--			break
--		end
--	end
--	return rtn
--end

--local function _SummarizeObjectives(self, achievementId)
--	-- TODO adapt to concrete line width!!!!
--	local numCriteria = GetAchievementNumCriteria(achievementId)
--	local lineText
--	local lines = 0
--	local maxLen = self.profile.maxAchievementLineLength
--	local maxLines = self.profile.maxAchievementLines
--	for i = 1, numCriteria do
--		local criteriaString, criteriaType, criteriaCompleted, quantity, totalQuantity, name, flags, assetID, quantityString, criteriaID, eligible = GetAchievementCriteriaInfo(achievementId, i)
--		if (not criteriaCompleted) then
--			if (lineText) then
--				if (strlen(lineText) + 2 + strlen(criteriaString) <= maxLen) then
--					lineText = lineText..", "..criteriaString
--				else
--					self:AddChildView(_GetText(self, lineText, description, self.profile.objectivePaddingLeft, self.profile.objectivePaddingTop))
--					lines = lines + 1
--					lineText = criteriaString
--				end
--			else
--				lineText = criteriaString
--			end
--			if (lines >= maxLines) then
--				if (lineText) then
--					lineText = nil
--					self:AddChildView(_GetText(self, "...", description, self.profile.objectivePaddingLeft, self.profile.objectivePaddingTop))
--				end
--				break
--			end
--		end
--	end
--	if (lineText) then
--		self:AddChildView(_GetText(self, lineText, description, self.profile.objectivePaddingLeft, self.profile.objectivePaddingTop))
--	end
--end

local function _SummarizeObjectives(self, achievementId)
	log:Debug("_SummarizeObjectives achievementId [%s]", achievementId)
	local numCriteria = GetAchievementNumCriteria(achievementId)
	local text
	local maxCriteriaLen = self.profile.maxAchievementCriteriaLength
	local maxLen = self.profile.maxAchievementTextLength
	for i = 1, numCriteria do
		local criteriaString, criteriaType, criteriaCompleted, quantity, totalQuantity, name, flags, assetID, quantityString, criteriaID, eligible = GetAchievementCriteriaInfo(achievementId, i)
		if (not criteriaCompleted) then
			local str = criteriaString
			if (strlen(str) > maxCriteriaLen) then
				str = string.sub(str, 1, maxCriteriaLen - 3).."..."
			end
			if (text) then
				text = text..", "..str
			else
				text = str
			end
			if (strlen(text) > maxLen) then
				text = string.sub(text, 1, maxLen - 3).."[...]"
				break
			end
		end
	end
	if (text) then
		log:Debug("add text [%s]", text)
		local code = YakaGui:GetFontColorCode(unpack(self.profile.objectiveColor))
		local text = code..text..FONT_COLOR_CODE_CLOSE
		self:AddChildView(_GetText(self, text, "fit", description, self.profile.objectivePaddingLeft, self.profile.objectivePaddingTop))
	end
end

local function _PrintObjective(self, achievementId)
	local numCriteria = GetAchievementNumCriteria(achievementId) -- should be one
	if (numCriteria == 1) then
		local criteriaString, criteriaType, criteriaCompleted, quantity, totalQuantity, name, flags, assetID, quantityString, criteriaID, eligible = GetAchievementCriteriaInfo(achievementId, 1)
		log:Debug("criteriaString [%s] criteriaType [%s] name [%s] quantity [%s] totalQuantity [%s] quantityString [%s]", criteriaString, criteriaType, name, quantity, totalQuantity, quantityString)
		if (not criteriaCompleted) then
			if (totalQuantity > 1 and self.profile.colorizeQuestObjectives) then
				local code = _ColorizeObjective(quantity, totalQuantity)
				local text = code..quantityString..FONT_COLOR_CODE_CLOSE
				self:AddChildView(_GetText(self, text, 12, description, self.profile.objectivePaddingLeft, self.profile.objectivePaddingTop))
			else
				local code = YakaGui:GetFontColorCode(unpack(self.profile.objectiveColor))
				local text = code..criteriaString..FONT_COLOR_CODE_CLOSE
				self:AddChildView(_GetText(self, criteriaString, 12, description, self.profile.objectivePaddingLeft, self.profile.objectivePaddingTop))
			end
		end
	end
end

local function _Update(self)
	-- reinit entry
	_WhipeChildViews(self.title)
	_WhipeChildViews(self)
	
	self.order = 1
	local achievementId = self.cfg.achievementId
	
	local id, name, points, completed, month, day, year, description, flags, image, rewardText = GetAchievementInfo(achievementId)
	
	-- add title
	local title = YakaGui:GetFontColorCode(unpack(self.profile.entryColor))..name..FONT_COLOR_CODE_CLOSE
	if (self.cfg.ignoreable) then
		self.title:AddChildView(_GetCheckbox(self, achievementId))
		self.title:AddChildView(_GetText(self, title, 12, description, self.profile.entryPaddingLeft, self.profile.entryPaddingTop))
		self:AddChildView(self.title)
	else
		self:AddChildView(_GetText(self, title, 12, description, self.profile.entryPaddingLeft, self.profile.entryPaddingTop))
	end

	-- add criteria
	local numCriteria = GetAchievementNumCriteria(achievementId)
	if (numCriteria > 1) then
		_SummarizeObjectives(self, achievementId)
	else
		_PrintObjective(self, achievementId)
	end
	

end

local function _Init(self, cfgIn, parent)
	self:InitCfg(cfgIn)
	
	local cfg = self.cfg
	self.profile = qmstr.QuestMaster.db.profile
	cfg.achievementId = cfgIn.achievementId
	cfg.ignoreable = cfgIn.ignoreable
	self.order = 1
	
	cfg.layout = {
		layoutType = "LinearLayout",
		vertical = true
	}
	cfg.paddingLeft = 34
	
	self.title = YakaGui:CreateView({
		viewType = "Container",
		order = 1,
		layout = {
			layoutType = "LinearLayout",
			vertical = false
		}
	})
	
	self:InitFrame(nil, parent)
	
	_Update(self)
end

--[[ 
	Creates a new instance.
--]]
function qmstr.AchievementEntry:new(cfgIn, parent)
	local instance = setmetatable({}, self.metatable)
	_Init(instance, cfgIn, parent)
	return instance
end

function qmstr.AchievementEntry.prototype:GetView()
	return self.view
end

function qmstr.AchievementEntry.prototype:SetAchievementId(achievementId, ignoreable)
	self.cfg.achievementId = achievementId
	self.cfg.ignoreable = ignoreable
	_Update(self)
end

-- register view

YakaGui:RegisterViewType(qmstr.AchievementEntry)