--[[
	Copyright (C) Udorn (Blackhand)
--]]

qmstr = qmstr or {}

qmstr.QuestWindow = {}
qmstr.QuestWindow.prototype = {}
qmstr.QuestWindow.metatable = {__index = qmstr.QuestWindow.prototype}

local YakaGui = LibStub("YakaGui")

local log = vendor.Debug:new("QuestWindow")

local BUTTON_SIZE = 16
local BOOK_TEX_COORDS = {0, 53/64, 0, 48/64}

local BUTTON_TEX_COORDS = {0, 19/32, 0, 18/32}
local BUTTON_WIDTH = 15
local BUTTON_HEIGHT = 14

-- TODO sound and flash on update

local function _OnCollapseClick(info)
	local self = info.arg
	
	self.profile.collapsed = not self.profile.collapsed
	self.frame:SetVisibility(not self.profile.collapsed)
end

local function _GetOrder(self)
	local order = self.order
	self.order = self.order + 1
	return order
end

local function _InitFrame(self)
	
	local frame = YakaGui:CreateView(self.cfg)
	frame:SetVisibility(true)
	
	self.questContainer = YakaGui:GetView("QuestMasterQuestContainer")
	self.questSelect = YakaGui:GetView("QuestMasterSelect")
	
	local collapseIcon = YakaGui:CreateView({
		viewType = "ToggleIcon",
		width = BUTTON_WIDTH,
		height = BUTTON_HEIGHT,
		--topLevel = true,
		--frameStrata = "HIGH",
		checkedNormalTexture = {
			file = "Interface\\Addons\\QuestMaster\\resources\\Button-Expand-Up",
			texCoords = BUTTON_TEX_COORDS
		},
		checkedPushedTexture = {
			file = "Interface\\Addons\\QuestMaster\\resources\\Button-Expand-Down",
			texCoords = BUTTON_TEX_COORDS
		},
		uncheckedNormalTexture = {
			file = "Interface\\Addons\\QuestMaster\\resources\\Button-Collapse-Up",
			texCoords = BUTTON_TEX_COORDS
		},
		uncheckedPushedTexture = {
			file = "Interface\\Addons\\QuestMaster\\resources\\Button-Collapse-Down",
			texCoords = BUTTON_TEX_COORDS
		},
		highlightTexture = {
			file = "Interface\\Addons\\QuestMaster\\resources\\Button-Highlight",
			alphaMode = "ADD"
		},
		setValue = _OnCollapseClick,
		getValue = function(info) return info.arg.profile.collapse end,
		arg = self
	})
	collapseIcon:SetPoint("TOPRIGHT", frame, "TOPRIGHT", -3, -4)
	collapseIcon:SetVisibility(true)
	
	self.frame = frame
	self.collapseIcon = collapseIcon
end

local function _WhipeChildViews(self, container)
	local questEntries = self.questEntries
	local questHeaders = self.questHeaders
	local selectors = self.selectors
	for k,v in pairs(container:GetChildViews()) do
		if (v:GetViewType() == "QuestEntry") then
			table.insert(questEntries, v)
		elseif (v:GetViewType() == "Text") then
			table.insert(questHeaders, v)
		elseif (v:GetViewType() == "QuestSelectorEntry") then
			table.insert(selectors, v)
		end
	end
	container:RemoveChildViews()
	self.order = 1
end

local function _GetQuestHeader(self, questIndex, text)
	local order = _GetOrder(self)
	if (#self.questHeaders > 0) then
		local header = table.remove(self.questHeaders)
		header:SetQuestIndex(questIndex, self.profile.filter)
		header:SetOrder(order)
		header:SetPadding(self.profile.headerPaddingLeft, 10, self.profile.headerPaddingTop, 0)
		return header
	end
	return YakaGui:CreateView({
		viewType = "QuestHeader",
		order = order,
		questIndex = questIndex,
		collapseable = self.profile.filter,
		height = 12,
		paddingLeft = self.profile.headerPaddingLeft,
		paddingRight = 10,
		paddingTop = self.profile.headerPaddingTop,
		font = "QuestMasterNormal"
	})
end

local function _GetQuestEntry(self, questIndex)
	local order = _GetOrder(self)
	if (#self.questEntries > 0) then
		local questEntry = table.remove(self.questEntries)
		questEntry:SetQuestIndex(questIndex, self.profile.filter)
		questEntry:SetOrder(order)
		return questEntry
	end
	return YakaGui:CreateView({
		viewType = "QuestEntry",
		order = order,
		questIndex = questIndex,
		paddingRight = 10,
		ignoreable = self.profile.filter
	})
end

local function _GetAchievementEntry(self, achievementId)
	local order = _GetOrder(self)
	if (#self.achievementEntries > 0) then
		local achievementEntry = table.remove(self.achivementEntries)
		achievementEntry:SetAchievementId(achievementId, self.profile.filter)
		achievementEntry:SetOrder(order)
		return achievementEntry
	end
	return YakaGui:CreateView({
		viewType = "AchievementEntry",
		order = order,
		achievementId = achievementId,
		paddingRight = 10,
		ignoreable = self.profile.filter
	})
end

local function _GetQuestSelectorEntry(self, questIndex)
	local order = _GetOrder(self)
	if (#self.selectors > 0) then
		local selector = table.remove(self.selectors)
		selector:SetQuestIndex(questIndex)
		selector:SetOrder(order)
		return selector
	end
	return YakaGui:CreateView({
		viewType = "QuestSelectorEntry",
		order = order,
		questIndex = questIndex
	})
end

local function _GetCurrentMapQuests(self)
	local numQuests = QuestMapUpdateAllQuests()
	local currentMapQuests = table.wipe(self.currentMapQuests)	
	for i = 1, numQuests do
		local questId = QuestPOIGetQuestIDByVisibleIndex(i)
		currentMapQuests[questId] = i
	end
end

local function _InitQuests(self)
	local quests = wipe(self.quests)
	local prevHeader
	local numEntries, numQuests = GetNumQuestLogEntries()
	
	-- check for newly completed quests
	local checkCompletedQuests = true
	local newCompletedQuests = false
	if (not self.completedQuests) then
		checkCompletedQuests = nil
		self.completedQuests = {}
	end
	local completedQuests = self.completedQuests
	
	for i = 1, numEntries do
		local view
		local questTitle, level, questTag, suggestedGroup, isHeader, isCollapsed, isComplete, isDaily, questID = GetQuestLogTitle(i)
		
		if (isComplete and not completedQuests[questID]) then
			newCompletedQuests = true
			completedQuests[questID] = true
		end
		
		if (isHeader) then
			local info = {isHeader = true, title = questTitle, questIndex = i, questId = questID, count = 0, isCollapsed = isCollapsed}
			prevHeader = info
			table.insert(quests, info)
		elseif (not qmstr.QuestMaster:IsQuestIgnored(questID) or self.profile.filter) then
			if (not self.profile.watchedQuestsOnly or IsQuestWatched(i) or self.profile.filter) then
				if (not self.profile.currentZoneOnly or self.currentMapQuests[questID] or self.profile.filter) then
    				local info = {title = questTitle, questIndex = i, questId = questID}
    				prevHeader.count = prevHeader.count + 1
    				table.insert(quests, info)
    			end
			end
		end
	end
	
	log:Debug("InitQuests checkCompletedQuests [%s] newCompletedQuests [%s]", checkCompletedQuests, newCompletedQuests)
	if (checkCompletedQuests and newCompletedQuests) then
		qmstr.QuestMaster:PlayQuestCompleteSound()
	end

	return quests
end

local function _UpdateAchievements(self, ...)
	log:Debug("_UpdateAchievements collapsed [%s]", self.profile.achievementCollapsed)
	if (self.profile.filter or not self.profile.achievementCollapsed) then
		local numTrackedAchievements = select("#", ...)
		if (numTrackedAchievements > 0) then
			-- select all not ignored achievements
			local achievements = wipe(self.achievements)
			for i = 1, numTrackedAchievements do
    			local achievementId = select(i, ...)
    			if (self.profile.filter or not qmstr.QuestMaster:IsAchievementIgnored(achievementId)) then
    				tinsert(achievements, achievementId)
    			end
			end
			if (#achievements > 0) then
				if (self.profile.showHeaders or self.profile.filter) then
        			self.achievementHeader:SetCollapseable(self.profile.filter)
        			self.questContainer:AddChildView(self.achievementHeader)
        		end
        		if (not self.profile.achievementCollapsed) then
            		for i = 1, #achievements do
            			local achievementId = achievements[i]
            			if (self.profile.filter or not qmstr.QuestMaster:IsAchievementIgnored(achievementId)) then
            				self.questContainer:AddChildView(_GetAchievementEntry(self, achievementId))
            			end
                	end
                end
            end
        end
	end
end

local function _UpdateQuests(self)
	if (self.profile.currentZoneOnly) then
		_GetCurrentMapQuests(self)
	end
	local quests = _InitQuests(self)
	for k,v in pairs(quests) do
		local view
		if (v.isHeader) then
			if ((not v.isCollapsed and v.count > 0) or self.profile.filter) then
				if (self.profile.showHeaders or self.profile.filter) then
					view = _GetQuestHeader(self, v.questIndex, "   "..v.title)
				end
			end
		elseif (self.profile.filter or not qmstr.QuestMaster:IsQuestIgnored(v.questId)) then
			view = _GetQuestEntry(self, v.questIndex)
		end
		
    	if (view) then
    		self.questContainer:AddChildView(view)
    	end
	end
	self.frame:RequestLayout()
end

--local function _UpdateSelectors(self)
--	_WhipeChildViews(self)
--
--	local numEntries, numQuests = GetNumQuestLogEntries()
--	if (numEntries > 0) then
--		for i = 1, numEntries do
--			local questTitle, level, questTag, suggestedGroup, isHeader = GetQuestLogTitle(i)
--			if (isHeader) then
--   				local view = _GetQuestSelectorEntry(self, i)
--    			self.questContainer:AddChildView(view)
--    		end
--		end
--	end
--	self.frame:RequestLayout()
--end

local function _UpdateLocked(self)
	if (self.profile.locked) then
		self.frame:SetBackdropColor({0, 0, 0, 0})
		self.frame:SetBackdropBorderColor({0, 0, 0, 0})
	else
    	self.frame:SetBackdropColor({0, 0, 0, 1}) 
    	self.frame:SetBackdropBorderColor({1, 1, 1, 1})
	end
	self.frame:SetMoveable(not self.profile.locked)
	self.frame:SetResizeable(not self.profile.locked)
end

local function _OnSelectClick(info)
	local self = info.arg
	self.profile.filter = info.value
	self:Update()
end

local function _Init(self)
	self.cfg = {
		viewType = "Window",
		frameStrata = "LOW",
		width = 320,
		height = 490,
		minWidth = 240,
		minHeight = 300,
		name = "QuestMaster",
		point = "TOPRIGHT",
		relativePoint = "BOTTOMRIGHT",
		relative = MinimapCluster,
		left = 0,
		top = 0,
		saveTable = self.profile.windows,
		layout = {
			layoutType = "LinearLayout",
			vertical = true,
		},
		childs = {
			buttons = {
				viewType = "Container",
				name = "QBarButtons",
				order = 1,
				paddingTop = 4,
				paddingRight = 18,
				layout = {
					layoutType = "LinearLayout",
					vertical = false
				},
				childs = {
					spacer = {
						viewType = "Spacer",
						order = 1,
						width = "fill",
						height = BUTTON_SIZE,
					},
					settings = {
                		viewType = "Icon",
                		order = 1.5,
                		width = BUTTON_WIDTH,
                		height = BUTTON_HEIGHT,
                		normalTexture = {
                			file = "Interface\\Addons\\QuestMaster\\resources\\Button-Settings-Up",
                			texCoords = BUTTON_TEX_COORDS
                		},
                		pushedTexture = {
                			file = "Interface\\Addons\\QuestMaster\\resources\\Button-Settings-Down",
                			texCoords = BUTTON_TEX_COORDS
                		},
                		highlightTexture = {
                			file = "Interface\\Addons\\QuestMaster\\resources\\Button-Highlight",
                			alphaMode = "ADD"
                		},
                		onClick = function() qmstr.Config:ToogleConfigDialog() end,
                		arg = self
                	},
					select = {
                		viewType = "ToggleIcon",
                		order = 2,
                		width = BUTTON_WIDTH,
                		height = BUTTON_HEIGHT,
                		checkedNormalTexture = {
                			file = "Interface\\Addons\\QuestMaster\\resources\\Button-Unfiltered-Up",
                			texCoords = BUTTON_TEX_COORDS
                		},
                		checkedPushedTexture = {
                			file = "Interface\\Addons\\QuestMaster\\resources\\Button-Unfiltered-Down",
                			texCoords = BUTTON_TEX_COORDS
                		},
                		uncheckedNormalTexture = {
                			file = "Interface\\Addons\\QuestMaster\\resources\\Button-Filter-Up",
                			texCoords = BUTTON_TEX_COORDS
                		},
                		uncheckedPushedTexture = {
                			file = "Interface\\Addons\\QuestMaster\\resources\\Button-Filter-Down",
                			texCoords = BUTTON_TEX_COORDS
                		},
                		highlightTexture = {
                			file = "Interface\\Addons\\QuestMaster\\resources\\Button-Highlight",
                			alphaMode = "ADD"
                		},
                		setValue = _OnSelectClick,
                		getValue = function(info) return info.arg.profile.filter end,
                		arg = self
                	}
				}
			},
			popups = {
		    	viewType = "QuestPopups",
		    	name = "QuestMasterPopups",
		    	order = 1.7,
    			width = "fill",
				height = 0,
				horizontalAlign = "RIGHT"
			},
		    scrollView = {
		    	viewType = "ScrollView",
		    	order = 2,
    			width = "fill",
				height = "fill",
				paddingTop = 4,
				--valueStep = 10, -- seems to be erroneous ?!?!
				scrollChild = {
					viewType = "Container",
					name = "QuestMasterQuestContainer",
					childs = {}
        		}
			},
			spacer = {
				viewType = "Spacer",
				order = 3,
				width = "fill",
				height = 5
			}
		}
	}
	self.questEntries = {}
	self.achievementEntries = {}
	self.questHeaders = {}
	self.selectors = {}
	self.quests = {}
	self.achievements = {}
	self.currentMapQuests = {}
	
	self.achievementHeader = YakaGui:CreateView({
		viewType = "AchievementHeader",
		order = 0,
		height = 12,
		width = "fill",
		paddingLeft = self.profile.headerPaddingLeft,
		paddingRight = 10,
		paddingTop = 1
	})
	
	_InitFrame(self)
	
	self.popups = YakaGui:GetView("QuestMasterPopups")
	
	self:Update()
end

--[[ 
	Creates a new instance.
--]]
function qmstr.QuestWindow:new()
	local instance = setmetatable({}, self.metatable)
	instance.profile = qmstr.QuestMaster.db.profile
	_Init(instance)
	return instance
end

function qmstr.QuestWindow.prototype:Update()
	if (self.isUpdating) then
		return
	end
	self.isUpdating = true
	
	self.achievementHeader:SetPadding(self.profile.headerPaddingLeft, 10, 1, 0)
	_UpdateLocked(self)
	_WhipeChildViews(self, self.questContainer)
	_UpdateAchievements(self, GetTrackedAchievements())
	_UpdateQuests(self)
	
	self.isUpdating = nil
end

function qmstr.QuestWindow.prototype:Destroy()
	self.frame:Destroy()
	self.collapseIcon:Destroy()
	self.frame = nil
	self.collapseIcon = nil
end

function qmstr.QuestWindow.prototype:DisplayAutoQuestPopUps(nextAnchor, maxHeight, frameWidth)
	local nextAnchor, maxLineWidth, numObjectives = self.popups:DisplayAutoQuestPopUps(nextAnchor, maxHeight, frameWidth)
	self.frame:RequestLayout()
	return nextAnchor, maxLineWidth, numObjectives
end